DEFAULT_STAGING_URL = "https://stgapi.betconnect.com/"
API_VERSION = "api/v2"
DEVELOPER_DOCS = "https://developer.betconnect.com/"
PAGE_START_VALUE = 1
MINIMUM_LIMIT_VALUE = 10

# TODO move these
MIN_CUSTOMER_STRATEGY_REF_LENGTH = 1
MAX_CUSTOMER_STRATEGY_REF_LENGTH = 15
MIN_CUSTOMER_ORDER_REF_LENGTH = 1
MAX_CUSTOMER_ORDER_REF_LENGTH = 36
CLIENT_READOUT_TIME_SECS = 120
LOGIN_FREQUENCY_CHECK_SECS = 60 * 120
CLIENT_TOKEN_REFRESH_FREQUENCY = 60 * 15
BET_REQUEST_MIN_ODDS = 1.01
SITE_MINIMUM_STAKE_SIZE = 1
